﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using AbstractLawFirmFileImplement.Models;

namespace AbstractLawFirmFileImpliment
{
	internal class DataFileSingleton
	{
		private static DataFileSingleton? instance;
		private readonly string ComponentFileName = "Component.xml";
		private readonly string OrderFileName = "Order.xml";
		private readonly string DocumentFileName = "Document.xml";
		private readonly string ShopFileName = "Shop.xml";

		public List<Component> Components { get; private set; }
		public List<Order> Orders { get; private set; }
		public List<Document> Documents { get; private set; }
		public List<Shop> Shops { get; private set; }
		public static DataFileSingleton GetInstance()
		{
			if (instance == null)
			{
				instance = new DataFileSingleton();
			}
			return instance;
		}
		public void SaveComponents() => SaveData(Components, ComponentFileName, "Components", x => x.GetXElement);
		public void SaveDocuments() => SaveData(Documents, DocumentFileName, "Documents", x => x.GetXElement);
		public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
		public void SaveShops() => SaveData(Shops, ShopFileName, "Shops", x => x.GetXElement);
		private DataFileSingleton()
		{
			Components = LoadData(ComponentFileName, "Component", x => Component.Create(x)!)!;
			Documents = LoadData(DocumentFileName, "Document", x => Document.Create(x)!)!;
			Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
			Shops = LoadData(ShopFileName, "Shop", x => Shop.Create(x)!)!;
		}
		private static List<T>? LoadData<T>(string filename, string xmlNodeName,
	   Func<XElement, T> selectFunction)
		{
			if (File.Exists(filename))
			{
				return
			   XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
			}
			return new List<T>();
		}
		private static void SaveData<T>(List<T> data, string filename, string
	   xmlNodeName, Func<T, XElement> selectFunction)
		{
			if (data != null)
			{
				new XDocument(new XElement(xmlNodeName, data.Select(selectFunction).ToArray())).Save(filename);
			}

		}
	}
}
