﻿using AbstractLawFirmContracts.BindingModels;
using AbstractLawFirmContracts.ViewModels;
using AbstractLawFirmDataModels.Models;
using AbstractLawFirmFileImpliment;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AbstractLawFirmFileImplement.Models
{
	public class Document : IDocumentModel
	{
		public int Id { get; private set; }
		public string DocumentName { get; private set; } = string.Empty;
		public double Price { get; private set; }
		public Dictionary<int, int> Components { get; private set; } = new();
		private Dictionary<int, (IComponentModel, int)>? _documentComponents = null;
		public Dictionary<int, (IComponentModel, int)> DocumentComponents
		{
			get
			{
				if (_documentComponents == null)
				{
					var source = DataFileSingleton.GetInstance();
					_documentComponents = Components.ToDictionary(x => x.Key, y =>
					((source.Components.FirstOrDefault(z => z.Id == y.Key) as IComponentModel)!,
					y.Value));
				}
				return _documentComponents;
			}
		}
		public static Document? Create(DocumentBindingModel model)
		{
			if (model == null)
			{
				return null;
			}
			return new Document()
			{
				Id = model.Id,
				DocumentName = model.DocumentName,
				Price = model.Price,
				Components = model.DocumentComponents.ToDictionary(x => x.Key, x
			   => x.Value.Item2)
			};
		}
		public static Document? Create(XElement element)
		{
			if (element == null)
			{
				return null;
			}
			return new Document()
			{
				Id = Convert.ToInt32(element.Attribute("Id")!.Value),
				DocumentName = element.Element("DocumentName")!.Value,
				Price = Convert.ToDouble(element.Element("Price")!.Value),
				Components = element.Element("DocumentComponents")!.Elements("DocumentComponent").ToDictionary(x =>
		   Convert.ToInt32(x.Element("Key")?.Value), x =>
		   Convert.ToInt32(x.Element("Value")?.Value))
			};
		}
		public void Update(DocumentBindingModel model)
		{
			if (model == null)
			{
				return;
			}
			DocumentName = model.DocumentName;
			Price = model.Price;
			Components = model.DocumentComponents.ToDictionary(x => x.Key, x => x.Value.Item2);
			_documentComponents = null;
		}
		public DocumentViewModel GetViewModel => new()
		{
			Id = Id,
			DocumentName = DocumentName,
			Price = Price,
			DocumentComponents = DocumentComponents
		};
		public XElement GetXElement => new("Document",
		new XAttribute("Id", Id),
		new XElement("DocumentName", DocumentName),
		new XElement("Price", Price.ToString()),
		new XElement("DocumentComponents", Components.Select(x =>
		new XElement("DocumentComponent", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()));
	}
}
