﻿using AbstractLawFirmContracts.BindingModels;
using AbstractLawFirmContracts.ViewModels;
using AbstractLawFirmDataModels.Enums;
using AbstractLawFirmDataModels.Models;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AbstractLawFirmFileImplement.Models
{
	public class Order : IOrderModel
	{
		public int DocumentId { get; private set; }

		public int Count { get; private set; }

		public double Sum { get; private set; }

		public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

		public DateTime DateCreate { get; private set; } = DateTime.Now;

		public DateTime? DateImplement { get; private set; }

		public int Id { get; private set; }

		public static Order? Create(OrderBindingModel? model)
		{
			if (model == null)
			{
				return null;
			}
			return new Order
			{
				DocumentId = model.DocumentId,
				Count = model.Count,
				Sum = model.Sum,
				Status = model.Status,
				DateCreate = model.DateCreate,
				DateImplement = model.DateImplement,
				Id = model.Id,
			};
		}

		public static Order? Create(XElement element)
		{
			if (element == null)
			{
				return null;
			}
			return new Order()
			{
				Id = Convert.ToInt32(element.Attribute("Id")!.Value),
				DocumentId = Convert.ToInt32(element.Element("DocumentId")!.Value),
				Sum = Convert.ToDouble(element.Element("Sum")!.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
				Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
				DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
				DateImplement = string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? null : Convert.ToDateTime(element.Element("DateImplement")!.Value)
			};
		}

		public void Update(OrderBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			Status = model.Status;
			DateImplement = model.DateImplement;
		}

		public OrderViewModel GetViewModel => new()
		{
			DocumentId = DocumentId,
			Count = Count,
			Sum = Sum,
			DateCreate = DateCreate,
			DateImplement = DateImplement,
			Id = Id,
			Status = Status,
		};

		public XElement GetXElement => new(
			"Order",
			 new XAttribute("Id", Id),
			 new XElement("DocumentId", DocumentId.ToString()),
			 new XElement("Count", Count.ToString()),
			 new XElement("Sum", Sum.ToString()),
			 new XElement("Status", Status.ToString()),
			 new XElement("DateCreate", DateCreate.ToString()),
			 new XElement("DateImplement", DateImplement.ToString())
		);
	}
}
