﻿using AbstractLawFirmContracts.BindingModels;
using AbstractLawFirmContracts.ViewModels;
using AbstractLawFirmDataModels.Models;
using AbstractLawFirmFileImpliment;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AbstractLawFirmFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; } = string.Empty;
        public string Address { get; private set; } = string.Empty;
        public int MaxCountDocuments { get; private set; }
        public DateTime OpeningDate { get; private set; }
        public Dictionary<int, int> Documents { get; private set; } = new();
        private Dictionary<int, (IDocumentModel, int)>? _shopDocuments = null;
        public Dictionary<int, (IDocumentModel, int)> ShopDocuments
        {
            get
            {
                if (_shopDocuments == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopDocuments = Documents.ToDictionary(
                        x => x.Key,
                        y => ((source.Documents.FirstOrDefault(z => z.Id == y.Key) as IDocumentModel)!, y.Value)
                    );
                }
                return _shopDocuments;
            }
        }
        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                MaxCountDocuments = model.MaxCountDocuments,
                OpeningDate = model.OpeningDate,
                Documents = model.ShopDocuments.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                MaxCountDocuments = Convert.ToInt32(element.Element("MaxCountDocuments")!.Value),
                OpeningDate = Convert.ToDateTime(element.Element("OpeningDate")!.Value),
                Documents = element.Element("ShopDocuments")!.Elements("ShopDocument").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value)
                )
            };
        }


        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            MaxCountDocuments = model.MaxCountDocuments;
            OpeningDate = model.OpeningDate;
            if (model.ShopDocuments.Count > 0)
            {
                Documents = model.ShopDocuments.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopDocuments = null;
            }
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            MaxCountDocuments = MaxCountDocuments,
            OpeningDate = OpeningDate,
            ShopDocuments = ShopDocuments,
        };

        public XElement GetXElement => new(
        "Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("MaxCountDocuments", MaxCountDocuments),
             new XElement("OpeningDate", OpeningDate.ToString()),
             new XElement("ShopDocuments", Documents.Select(x =>
             new XElement("ShopDocument",
             new XElement("Key", x.Key),
             new XElement("Value", x.Value)))
            .ToArray()));
    }
}