﻿using AbstractLawFirmContracts.BindingModels;
using AbstractLawFirmContracts.BindingModels.BindingModels;
using AbstractLawFirmContracts.SearchModels;
using AbstractLawFirmContracts.StoragesContracts;
using AbstractLawFirmContracts.ViewModels;
using AbstractLawFirmListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractLawFirmListImplement.Implements
{
    public class DocumentStorage : IDocumentStorage
    {
        private readonly DataListSingleton _source;
        public DocumentStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<DocumentViewModel> GetFullList()
        {
            var result = new List<DocumentViewModel>();
            foreach (var document in _source.Documents)
            {
                result.Add(document.GetViewModel);
            }
            return result;
        }
        public List<DocumentViewModel> GetFilteredList(DocumentSearchModel
       model)
        {
            var result = new List<DocumentViewModel>();
            if (string.IsNullOrEmpty(model.DocumentName))
            {
                return result;
            }
            foreach (var document in _source.Documents)
            {
                if (document.DocumentName.Contains(model.DocumentName))
                {
                    result.Add(document.GetViewModel);
                }
            }
            return result;
        }
        public DocumentViewModel? GetElement(DocumentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DocumentName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var document in _source.Documents)
            {
                if ((!string.IsNullOrEmpty(model.DocumentName) &&
               document.DocumentName == model.DocumentName) ||
                (model.Id.HasValue && document.Id == model.Id))
                {
                    return document.GetViewModel;
                }
            }
            return null;
        }
        public DocumentViewModel? Insert(DocumentBindingModel model)
        {
            model.Id = 1;
            foreach (var document in _source.Documents)
            {
                if (model.Id <= document.Id)
                {
                    model.Id = document.Id + 1;
                }
            }
            var newDocument = Document.Create(model);
            if (newDocument == null)
            {
                return null;
            }
            _source.Documents.Add(newDocument);
            return newDocument.GetViewModel;
        }
        public DocumentViewModel? Update(DocumentBindingModel model)
        {
            foreach (var document in _source.Documents)
            {
                if (document.Id == model.Id)
                {
                    document.Update(model);
                    return document.GetViewModel;
                }
            }
            return null;
        }
        public DocumentViewModel? Delete(DocumentBindingModel model)
        {
            for (int i = 0; i < _source.Documents.Count; ++i)
            {
                if (_source.Documents[i].Id == model.Id)
                {
                    var element = _source.Documents[i];
                    _source.Documents.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
